@@ -263,7 +263,7 @@ swf_tag_get_jpeg_data(swf_tag_t *tag, unsigned long *length, int image_id, swf_t
         swf_tag_create_detail(tag, NULL);
     }
     if (! tag->detail) {
-        fprintf(stderr, "Can't create tag\n");
+        fprintf(stderr, "swf_tag_get_jpeg_data: Can't create tag\n");
         return NULL;
     }
     if (tag_jpegtables) {
@@ -300,7 +300,7 @@ swf_tag_get_alpha_data(swf_tag_t *tag, unsigned long *length, int image_id) {
         swf_tag_create_detail(tag, NULL);
     }
     if (! tag->detail) {
-        fprintf(stderr, "Can't create tag\n");
+        fprintf(stderr, "swf_tag_get_alpha_data: Can't create tag\n");
         return NULL;
     }
     return swf_tag_jpeg_get_alpha_data(tag->detail, length, image_id);
@@ -334,7 +334,7 @@ swf_tag_replace_jpeg_data(swf_tag_t *tag, int image_id,
         swf_tag_create_detail(tag, NULL);
     }
     if (! tag->detail) {
-        fprintf(stderr, "Can't create tag\n");
+        fprintf(stderr, "swf_tag_replace_jpeg_data: Can't create tag\n");
         return 1;
     }
     result= swf_tag_jpeg_replace_jpeg_data(tag->detail, image_id,
@@ -364,7 +364,7 @@ swf_tag_get_png_data(swf_tag_t *tag, unsigned long *length, int image_id) {
         swf_tag_create_detail(tag, NULL);
     }
     if (! tag->detail) {
-        fprintf(stderr, "Can't create tag\n");
+        fprintf(stderr, "swf_tag_get_png_data: Can't create tag\n");
         return NULL;
     }
     return swf_tag_lossless_get_png_data(tag->detail, length, image_id, tag);
@@ -396,7 +396,7 @@ swf_tag_replace_png_data(swf_tag_t *tag, int image_id,
         swf_tag_create_detail(tag, NULL);
     }
     if (! tag->detail) {
-        fprintf(stderr, "Can't create tag\n");
+        fprintf(stderr, "swf_tag_replace_png_data: Can't create tag\n");
         return 1;
     }
     result= swf_tag_lossless_replace_png_data(tag->detail, image_id,
@@ -409,6 +409,71 @@ swf_tag_replace_png_data(swf_tag_t *tag, int image_id,
     return result;
 }
 
+/*
+ * DefineSound
+ */
+
+unsigned char *
+swf_tag_get_sound_data(swf_tag_t *tag, unsigned long *length, int sound_id) {
+    swf_tag_info_t *tag_info;
+    *length = 0;
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_get_sound_data: tag == NULL\n");
+        return NULL;
+    }
+    tag_info = get_swf_tag_info(tag->tag);
+    if (tag->tag != 14) { // DefineSound
+        return NULL;
+    }
+    if (! tag->detail) {
+        swf_tag_create_detail(tag, NULL);
+    }
+    if (! tag->detail) {
+        fprintf(stderr, "swf_tag_get_sound_data: Can't create tag detail\n");
+        return NULL;
+    }
+    return swf_tag_sound_get_sound_data(tag->detail, length, sound_id);
+}
+
+int
+swf_tag_replace_melo_data(swf_tag_t *tag, int sound_id,
+                          unsigned char *melo_data,
+                          unsigned long melo_data_len) {
+    swf_tag_info_t *tag_info;
+    int result;
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_replace_melo_data: tag == NULL\n");
+        return 1;
+    }
+    if (tag->tag != 14) { // DefineSound
+        return 1;
+    }
+    tag_info = get_swf_tag_info(tag->tag);
+    if (tag_info && tag_info->detail_handler) {
+        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
+        if (detail_handler->identity) {
+            if (detail_handler->identity(tag->data, sound_id, tag)) {
+                return 1;
+            }
+        }
+    }
+    if (! tag->detail) {
+        swf_tag_create_detail(tag, NULL);
+    }
+    if (! tag->detail) {
+        fprintf(stderr, "swf_tag_replace_melog_data: Can't create tag\n");
+        return 1;
+    }
+    result= swf_tag_sound_replace_melo_data(tag->detail, sound_id,
+                                            melo_data, melo_data_len);
+    if (result == 0) {
+        free(tag->data);
+        tag->data = NULL;
+        tag->length = 0;
+    }
+    return result;
+}
+
 char *
 swf_tag_get_edit_string(swf_tag_t *tag,
                         char *variable_name, int variable_name_len,
