@@ -266,9 +266,14 @@ PHP_METHOD(swfed, output) {
     swf = get_swf_object(getThis());
     data = swf_object_output(swf, &len);
     new_buff = emalloc(len);
+    if (new_buff == NULL) {
+        fprintf(stderr, "output: Can't emalloc new_buff\n");
+        free(data);
+        RETURN_FALSE;
+    }
     memcpy(new_buff, data, len);
     free(data);
-    RETURN_STRINGL(new_buff, len, 1);
+    RETURN_STRINGL(new_buff, len, 0);
 }
 
 PHP_METHOD(swfed, getHeaderInfo) {
@@ -456,13 +461,13 @@ PHP_METHOD(swfed, getJpegData) {
     }
     new_buff = emalloc(len);
     if (new_buff == NULL) {
-        fprintf(stderr, "getJpegAlpha Can't emalloc new_buff\n");
+        fprintf(stderr, "getJpegData Can't emalloc new_buff\n");
         free(data);
         RETURN_FALSE;
     }
     memcpy(new_buff, data, len);
     free(data);
-    RETURN_STRINGL(new_buff, (int) len, 1);
+    RETURN_STRINGL(new_buff, (int) len, 0);
 }
 
 PHP_METHOD(swfed, getJpegAlpha) {
@@ -481,9 +486,14 @@ PHP_METHOD(swfed, getJpegAlpha) {
         RETURN_FALSE;
     }
     new_buff = emalloc(len);
+    if (new_buff == NULL) {
+        fprintf(stderr, "getJpegAlpha Can't emalloc new_buff\n");
+        free(data);
+        RETURN_FALSE;
+    }
     memcpy(new_buff, data, len);
     free(data);
-    RETURN_STRINGL(new_buff, len, 1);
+    RETURN_STRINGL(new_buff, len, 0);
 }
 
 PHP_METHOD(swfed, replaceJpegData) {
@@ -542,7 +552,7 @@ PHP_METHOD(swfed, getPNGData) {
     }
     memcpy(new_buff, data, len);
     free(data);
-    RETURN_STRINGL(new_buff, (int) len, 1);
+    RETURN_STRINGL(new_buff, (int) len, 0);
 }
 
 PHP_METHOD(swfed, replacePNGData) {
@@ -594,7 +604,7 @@ PHP_METHOD(swfed, getSoundData) {
     }
     memcpy(new_buff, data, len);
     free(data);
-    RETURN_STRINGL(new_buff, (int) len, 1);
+    RETURN_STRINGL(new_buff, (int) len, 0);
 }
 
 PHP_METHOD(swfed, replaceMLDData) {
@@ -647,7 +657,7 @@ PHP_METHOD(swfed, getEditString) {
     }
     memcpy(new_buff, data, str_len + 1);
     free(data);
-    RETURN_STRINGL(new_buff, str_len, 1);
+    RETURN_STRINGL(new_buff, str_len, 0);
 }
 
 PHP_METHOD(swfed, replaceEditString) {
@@ -692,7 +702,7 @@ PHP_METHOD(swfed, getActionData) {
         RETURN_FALSE;
     }
     memcpy(new_buff, data_ref, data_len);
-    RETURN_STRINGL(new_buff, data_len, 1);
+    RETURN_STRINGL(new_buff, data_len, 0);
 }
 
 PHP_METHOD(swfed, disasmActionData) {
